Needs["xAct`xTensor`"];
Needs["xAct`xCoba`"];

(*$Post = Validate;*)
$PrePrint = ScreenDollarIndices;
SetOptions[ToCanonical,UseMetricOnVBundle->None];
SetOptions[ContractMetric, AllowUpperDerivatives -> False];




(********* constants **********)
DefConstantSymbol /@ {C1,C2,C3,C4,C5,C6,C7};
DefConstantSymbol /@ {B1,B2,B3,B4,B5,B6};
DefConstantSymbol /@ {x1,x2,x3,x4,x5,x6,x7,x8,x9,x10,x11,x12,x13,x14,x15,x16,x17,x18,x19,x20,x21};
DefConstantSymbol /@ {sTFT,sV,sV1,sV2,sV3,s11,s12,s13,s14,s22,s23,s24,s33,s34,s44};
DefConstantSymbol /@ {p1,p2,p3,p4};
DefConstantSymbol /@ {g111,g112,g113,g114,g115,g116,g117,g118,g119,g1110,g1111,g121,g122,g123,g221,g331,g332,g333,g334};
DefConstantSymbol[lambdaP, PrintAs->"\[Lambda]P2"];
DefConstantSymbol[lambdaM, PrintAs->"\[Lambda]M2"];
DefConstantSymbol[lambdaV, PrintAs->"\[Lambda]V2"];

(********** 3-manifold, time and coordinate basis ************)
DefManifold[M3,3,{i,j,k,l,m,i1,i2,i3,j1,j2,j3,k1,k2,k3,l1,l2,l3,m1,m2,m3}];
DefParameter[t];

DefBasis[cart,TangentM3,{1,2,3},BasisColor->RGBColor[0,0,1]]

(********** metric and related stuff **********)
DefMetric[1, gamma[-i, -j], D3, {"|", "D"}, PrintAs -> "\[Gamma]", WeightedWithBasis -> AIndex];
DependenciesOfTensor[gamma] ^= Join[DependenciesOfTensor[gamma],{t}];
DependenciesOfTensor[Detgamma] ^= Join[DependenciesOfTensor[Detgamma],{t}];

(***** Define spatial vector s *****)
DefTensor[s[-i], M3];

(***** s must satisfy several conditions *****)
s/:s[i_]s[-i_]:=1;
s/:s[Dir[s[_]]]:=1;

(***** Define the projection operator q *****)
DefTensor[q[i,j],M3,Symmetric[{1,2}]]

(***** q has trace 2 *****)
q/:q[i_,-i_]:=2;	q/:q[-i_,i_]:=2;

(***** is automatically contracted with itself *****)
q/:q[i_,j_]q[-j_,k_]:=q[i,k];	q/:q[i_,j_]q[k_,-j_]:=q[i,k]
q/:q[j_,i_]q[-j_,k_]:=q[i,k];	q/:q[j_,i_]q[k_,-j_]:=q[i,k];

(***** is orthogonal to s *****)
q/:q[i_,j_]s[-j_]:=0;	q/:q[i_,-j_]s[j_]:=0;
q/:q[j_,i_]s[-j_]:=0;	q/:q[-j_,i_]s[j_]:=0;


(***** orthonormal spatial basis ****)
DefTensor[V1[-i], {M3}, PrintAs->"V1"];
DefTensor[V2[-i], {M3}, PrintAs->"V2"];

AutomaticRules[V1, MakeRule[{V1[-i]V1[i],1}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{gamma[-i,-j] V1[i] V1[j], 1}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{V2[-i]V2[i],1}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{gamma[-i,-j] V2[i] V2[j], 1}, MetricOn->All]];
AutomaticRules[s, MakeRule[{s[-i]V1[i],0}, MetricOn->All]];
AutomaticRules[s, MakeRule[{gamma[-i,-j] s[i] V1[j], 0}, MetricOn->All]];
AutomaticRules[s, MakeRule[{s[-i]V2[i],0}, MetricOn->All]];
AutomaticRules[s, MakeRule[{gamma[-i,-j] s[i] V2[j], 0}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{V1[-i]V2[i],0}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{gamma[-i,-j] V1[i] V2[j], 0}, MetricOn->All]];

AutomaticRules[V1, MakeRule[{V1[-i]q[i,j],V1[j]}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{V1[i]q[-i,-j],V1[-j]}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{V1[-i]q[i,-j],V1[-j]}, MetricOn->All]];
AutomaticRules[V1, MakeRule[{V1[i]q[-i,j],V1[j]}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{V2[-i]q[i,j],V2[j]}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{V2[i]q[-i,-j],V2[-j]}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{V2[-i]q[i,-j],V2[-j]}, MetricOn->All]];
AutomaticRules[V2, MakeRule[{V2[i]q[-i,j],V2[j]}, MetricOn->All]];






(******* the fields ***********)
(******** canonical variables ***********)
DefTensor[pi[i, j], {M3,t}, Symmetric[{1, 2}], PrintAs->"\[CapitalPi]", WeightOfTensor->1 AIndex];
DefTensor[sigma[], {M3,t}, PrintAs->"\[Sigma]", WeightOfTensor->1 AIndex];
DefTensor[alpha[], {M3,t}, PrintAs->"\[Alpha]"];
DefTensor[rho[-k], {M3,t}, PrintAs->"\[Rho]", WeightOfTensor->1 AIndex];
DefTensor[beta[k], {M3,t}, PrintAs->"\[Beta]"];

(******** Z4 variables ***********)
DefTensor[K[-i,-j],{M3,t},Symmetric[{1,2}]];
DefTensor[Theta[],{M3,t},PrintAs->"\[Theta]"];
DefTensor[Z[-k],{M3,t},PrintAs->"Z"];

(******* constraints *********)
DefTensor[HamC[], {M3,t}, PrintAs->"Ht", WeightOfTensor->2 AIndex];	
DefTensor[MomC[-k], {M3,t}, PrintAs->"Hi", WeightOfTensor->1 AIndex];

(******** Hamiltonians *********)
DefTensor[HS[], {M3,t}, PrintAs->"HS", WeightOfTensor->1 AIndex];
DefTensor[Hadvection[], {M3,t}, PrintAs->"Hadv", WeightOfTensor->1 AIndex];
DefTensor[HgaugeGHG[], {M3,t}, WeightOfTensor->1 AIndex];
DefTensor[Hgauge[], {M3,t}, WeightOfTensor->1 AIndex];


(****** scalar functions ******)
DefScalarFunction[muL, PrintAs -> "\[Mu]L"];
DefScalarFunction[muS, PrintAs -> "\[Mu]S"];


(******* rules for change of variables ********)
(********** Z4 -> canonical variables **********)
RuleKtoPi        := MakeRule[{K[-i,-j],
                              -1/Sqrt[Detgamma[]] *
                              (gamma[-l,-j] gamma[-i,-m] - gamma[-l,-m] gamma[-i,-j]/2) pi[l,m]},
                             MetricOn->All];
RuleThetatoSigma := MakeRule[{Theta[], alpha[] sigma[]/2/Sqrt[Detgamma[]]},MetricOn->All];
RuleZtoRho       := MakeRule[{Z[-k], -alpha[] rho[-k]/2/Sqrt[Detgamma[]]},MetricOn->All];
RuleZ4toCanon    := Flatten[{RuleKtoPi,RuleThetatoSigma,RuleZtoRho}];

(********** canonical -> Z4 variables **********)
RulePitoK        := MakeRule[{pi[i,j],
                              Sqrt[Detgamma[]] *
                              (gamma[l,m] gamma[i,j] - gamma[i,m] gamma[l,j])K[-l,-m]},
                             MetricOn->All];
RuleSigmatoTheta := MakeRule[{sigma[],2 Theta[]/alpha[] Sqrt[Detgamma[]]},MetricOn->All];
RuleRhotoZ       := MakeRule[{rho[-k],-2 Z[-k]/alpha[] Sqrt[Detgamma[]]},MetricOn->All];
RuleCanontoZ4    := Flatten[{RulePitoK,RuleSigmatoTheta,RuleRhotoZ}];




(********* implicit tensor dependencies ***********)

xAct`xTensor`Private`ImplicitTensorDepQ[RiemannD3,gamma] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[RicciD3,gamma] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[RicciScalarD3,gamma] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[ChristoffelD3,gamma] := True;

xAct`xTensor`Private`ImplicitTensorDepQ[K,gamma] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[K,pi] := True;

xAct`xTensor`Private`ImplicitTensorDepQ[Theta,alpha] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[Theta,sigma] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[Theta,gamma] := True;

xAct`xTensor`Private`ImplicitTensorDepQ[Z,alpha] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[Z,rho] := True;
xAct`xTensor`Private`ImplicitTensorDepQ[Z,gamma] := True;





(********* abbreviations for VarD ***********)
(*** scalar functions with more than one parameter ***)
VarD[tensor_, der_][f_?ScalarFunctionQ[expr__], rest_] :=
Plus @@ (VarD[tensor, der][ReplaceDummies /@ {expr}[[#]],
                           (Derivative @@ (UnitVector[Length[{expr}], #]))[f][expr]*rest] & /@
                           Range[Length[{expr}]]);

(*** curvature in terms of the metric ***)
RicciScalarD3 /: VarD[gamma[i_, j_], PD ][ RicciScalarD3[], rest_] :=
Module[{k,l},VarD[gamma[i, j], PD ][RicciD3[-k, -l], gamma[k, l] rest]+
             VarD[gamma[i, j], PD ][gamma[k, l], RicciD3[-k, -l] rest]];
RicciD3 /: VarD[gamma[i_, j_], PD ][ RicciD3[-k_, -l_], rest_] :=
Module[{m,m1},VarD[gamma[i, j], PD ][ RiemannD3[m, -k, -m1, -l], delta[-m, m1] rest]];
RiemannD3 /: VarD[gamma[i_, j_], PD ][ RiemannD3[k_, -l_, -m_, -m1_], rest_] :=
VarD[gamma[i, j], PD ][ChristoffelD3[k, -m, -l], D3[-m1][rest]] -
VarD[gamma[i, j], PD ][ChristoffelD3[k, -m1, -l], D3[-m][rest]];

(*** Christoffel in terms of the metric ***)
ChristoffelD3 /: VarD[gamma[i_, j_], PD ][ ChristoffelD3[k_, -l_, -m_], rest_] :=
Module[{m1},VarD[gamma[i, j], PD ][ gamma[-l,-m], 1/2 gamma[k, m1] D3[-m1][rest]] -
            VarD[gamma[i, j], PD ][ gamma[-m1,-m], 1/2 gamma[k, m1] D3[-l][rest]] -
            VarD[gamma[i, j], PD ][ gamma[-l,-m1], 1/2 gamma[k, m1] D3[-m][rest]]];

(*** Det of metric ***)
Detgamma /: VarD[gamma[a_, b_], PD ][ Detgamma[], rest_] :=
Module[{i, j}, VarD[gamma[a, b], PD ][gamma[ -i, -j], Detgamma[] rest gamma[i, j]]];

(**** automatically replace time derivatives with wrong index position ****)
AutomaticRules[pi, MakeRule[{ParamD[t][pi[-i, -j]],
                             Evaluate@SeparateMetric[][ParamD[t][pi[-i, -j]]]}, MetricOn->None]];
AutomaticRules[pi, MakeRule[{ParamD[t][pi[i, -j]],
                             Evaluate@SeparateMetric[][ParamD[t][pi[i, -j]]]}, MetricOn->None]];
AutomaticRules[pi, MakeRule[{ParamD[t][pi[-i, j]],
                             Evaluate@SeparateMetric[][ParamD[t][pi[-i, j]]]}, MetricOn->None]];
AutomaticRules[rho, MakeRule[{ParamD[t][rho[i]],
                                Evaluate@SeparateMetric[][ParamD[t][rho[i]]]}, MetricOn->None]];
AutomaticRules[beta, MakeRule[{ParamD[t][beta[-i]],
                               Evaluate@SeparateMetric[][ParamD[t][beta[-i]]]}, MetricOn->None]];






(********** bases for expansions of matrices ************)
derivABBasis[i_] :=
{{s[i], V1[i], V2[i]},
 {s[i], V1[i], V2[i]},
 {s[i], V1[i], V2[i]}};

fieldABBasis[i_, j_] :=
{{(V1[i]V2[j]+V2[i]V1[j])/Sqrt[2],
  (V1[i]V1[j]-V2[i]V2[j])/Sqrt[2],
  (s[i]V1[j]+s[j]V1[i])/Sqrt[2],
  (s[i]V2[j]+s[j]V2[i])/Sqrt[2],
  (V1[i]V1[j]+V2[j]V2[i])/Sqrt[2],
  s[i]s[j]},
 {1},
 {s[-i], V1[-i], V2[-i]}
};
fieldABBasis2[i_, j_] :=
{{(V1[i]V2[j]+V2[i]V1[j])/Sqrt[2],
  (V1[i]V1[j]-V2[i]V2[j])/Sqrt[2],
  (s[i]V1[j]+s[j]V1[i])/Sqrt[2],
  (s[i]V2[j]+s[j]V2[i])/Sqrt[2],
  (V1[i]V1[j]+V2[j]V2[i])/Sqrt[2],
  s[i]s[j]},
 {1},
 {s[i], V1[i], V2[i]}
};

projBasis[k_, l_, i_, j_] :=
{{Symmetrize[q[k,i] q[l,j], {i,j}] - 1/2 q[k,l] q[i,j],
  Sqrt[2] Symmetrize[s[k] q[l,i], {k,l}],
  q[k,l]/Sqrt[2],
  s[k] s[l]},
 {1},
 {s[-k],
  q[-i,-k]}};
projBasis2[k_, l_, i_, j_] :=
{{Symmetrize[q[k,i] q[l,j], {i,j}] - 1/2 q[k,l] q[i,j],
  Sqrt[2] Symmetrize[s[k] q[l,i], {k,l}],
  q[k,l]/Sqrt[2],
  s[k] s[l]},
 {1},
 {s[k],
  q[i,k]}};






(****** candidate symmetrizers ******)

(*** ansatz for a candidate of the fully second order system ***)

DefTensor[G11[i,j,k,l,m,m1], M3, GenSet[Images[{2, 1, 5, 6, 3, 4}], Cycles[{3, 4}], Cycles[{5, 6}]]];
DefTensor[G12[i,j,k,l], M3, GenSet[Cycles[{3, 4}]]];
DefTensor[G22[i,j], M3, Symmetric[{1,2}]];
DefTensor[G33[i,j,-k,-m], M3, GenSet[Images[{2, 1, 4, 3}]]];
DefTensor[G16[i,k,l,-m], M3, GenSet[Cycles[{2, 3}]]];
DefTensor[G26[i,-m], M3];
DefTensor[G34[i,-k,m,m1], M3, GenSet[Cycles[{3, 4}]]];
DefTensor[G35[i,-k], M3];
DefTensor[G44[k,l,m,m1], M3, GenSet[Images[{3, 4, 1, 2}], Cycles[{1, 2}], Cycles[{3, 4}]]];
DefTensor[G45[k,l], M3, GenSet[Cycles[{1, 2}]]];
DefTensor[G55[], M3];
DefTensor[G66[-k,-m], M3, Symmetric[{1,2}]];

ansatzCand[i_,j_,k_,l_,m_,m1_] :=
{{G11[i, j, k, l, m, m1], G12[i, j, k, l], 0, 0, 0, G16[i, k, l, -m]},
 {G12[j, i, m, m1], G22[i, j], 0, 0, 0, G26[i, -m]},
 {0, 0, G33[i, j, -k, -m], G34[i, -k, m, m1], G35[i, -k], 0},
 {0, 0, G34[j, -m, k, l], G44[k, l, m, m1], G45[k, l], 0},
 {0, 0, G35[j, -m], G45[m, m1], G55[], 0},
 {G16[j, m, m1, -k], G26[j, -k], 0, 0, 0, G66[-k, -m]}};




(*** ansatz for a candidate of the FOITSOIS system in Z4 variables ***)

DefTensor[G11Z4[i,j,k,l,m,m1], M3, GenSet[Images[{2, 1, 5, 6, 3, 4}], Cycles[{3, 4}], Cycles[{5, 6}]]]; (*171 components*)
DefTensor[G12Z4[i,j,k,l], M3, GenSet[Cycles[{3, 4}]]]; (*54 components*)
DefTensor[G22Z4[i,j], M3, Symmetric[{1,2}]]; (*6 components*)
DefTensor[G33Z4[i,j,-k,-m], M3, GenSet[Images[{2, 1, 4, 3}]]]; (*45 components*)
DefTensor[G16Z4[i,k,l,m], M3, GenSet[Cycles[{2, 3}]]]; (*54 components*)
DefTensor[G26Z4[i,m], M3]; (*9 components*)
DefTensor[G34Z4[i,-k,m,m1], M3, GenSet[Cycles[{3, 4}]]]; (*54 components*)
DefTensor[G35Z4[i,-k], M3]; (*9 components*)
DefTensor[G44Z4[k,l,m,m1], M3, GenSet[Images[{3, 4, 1, 2}], Cycles[{1, 2}], Cycles[{3, 4}]]]; (*21 components*)
DefTensor[G45Z4[k,l], M3, GenSet[Cycles[{1, 2}]]]; (*6 components*)
DefTensor[G55Z4[], M3]; (*1 component*)
DefTensor[G66Z4[k,m], M3, Symmetric[{1,2}]]; (*6 components*)
(*total number of components: 436*)

ansatzCandZ4[i_,j_,k_,l_,m_,m1_] :=
{{G11Z4[i, j, k, l, m, m1], G12Z4[i, j, k, l], 0, 0, 0, G16Z4[i, k, l, m]},
 {G12Z4[j, i, m, m1], G22Z4[i, j], 0, 0, 0, G26Z4[i, m]},
 {0, 0, G33Z4[i, j, -k, -m], G34Z4[i, -k, m, m1], G35Z4[i, -k], 0},
 {0, 0, G34Z4[j, -m, k, l], G44Z4[k, l, m, m1], G45Z4[k, l], 0},
 {0, 0, G35Z4[j, -m], G45Z4[m, m1], G55Z4[], 0},
 {G16Z4[j, m, m1, k], G26Z4[j, k], 0, 0, 0, G66Z4[k, m]}};



(*** generic form of the product candidate.PPMatrix for FOITSOIS system in Z4 variables ***)
DefTensor[HFAProd31[i1,i2,i3,-k,m,m1], M3,
          GenSet[Images[{1,2,3,4,6,5}], Images[{1,3,2,4,5,6}], Images[{2,1,3,4,5,6}], Images[{3,2,1,4,5,6}]]];
DefTensor[HFAProd41[i1,i2,k,l,m,m1], M3,
          GenSet[Images[{1,2,3,4,6,5}], Images[{2,1,3,4,5,6}], Images[{1,2,4,3,5,6}]]];
DefTensor[HFAProd51[i1,i2,m,m1], M3,
          GenSet[Images[{2,1,3,4}], Images[{1,2,4,3}]]];
DefTensor[HFAProd32[i1,i2,i3,-k], M3,
          GenSet[Images[{1,3,2,4}], Images[{2,1,3,4}], Images[{3,2,1,4}]]];
DefTensor[HFAProd42[i1,i2,k,l], M3,
          GenSet[Images[{1,2,4,3}], Images[{2,1,3,4}]]];
DefTensor[HFAProd52[i1,i2], M3,
          GenSet[Images[{2,1}]]];
DefTensor[HFAProd63[i1,i2,k,-m], M3,
          GenSet[Images[{2,1}]]];
DefTensor[HFAProd64[i1,k,m,m1], M3,
          GenSet[Images[{1,2,4,3}]]];
DefTensor[HFAProd65[i1,k], M3];





DefConstantSymbol[G551];

DefConstantSymbol[G221];
DefConstantSymbol[G261];
DefConstantSymbol[G351];
DefConstantSymbol[G451];
DefConstantSymbol[G661];

DefConstantSymbol[G121];
DefConstantSymbol[G122];
DefConstantSymbol[G331];
DefConstantSymbol[G332];
DefConstantSymbol[A331];
DefConstantSymbol[G161];
DefConstantSymbol[G162];
DefConstantSymbol[G341];
DefConstantSymbol[G342];
DefConstantSymbol[G441];
DefConstantSymbol[G442];

DefConstantSymbol[G111];
DefConstantSymbol[G112];
DefConstantSymbol[G113];
DefConstantSymbol[G114];
DefConstantSymbol[A111];

makeMetricAnsatz2nd :=
Module[{},
ansatz2ndMade = True;

IndexSet[G55[], G551];

IndexSet[G22[i_,j_],G221 gamma[i,j]];
IndexSet[G26[i_,m_],G261 gamma[i,m]];
IndexSet[G35[i_,k_],G351 gamma[i,k]];
IndexSet[G45[k_,l_],G451 gamma[k,l]];
IndexSet[G66[k_,m_],G661 gamma[k,m]];

IndexSet[G12[i_,j_,k_,l_],(G121 (gamma[i,k] gamma[j,l] + gamma[i,l] gamma[j,k]) +
                           G122 gamma[i,j] gamma[k,l])];
IndexSet[G33[i_,j_,k_,m_],(G331 (gamma[i,k] gamma[j,m] + gamma[i,m] gamma[j,k]) +
                           G332 gamma[i,j] gamma[k,m]) +
                          (A331 (gamma[i,k] gamma[j,m] - gamma[i,m] gamma[j,k]))];
IndexSet[G16[i_,k_,l_,m_],(G161 (gamma[i,k] gamma[m,l] + gamma[m,k] gamma[i,l]) +
                           G162 gamma[i,m] gamma[k,l])];
IndexSet[G34[i_,k_,m_,m1_], G341 (gamma[i,m] gamma[k,m1] + gamma[i,m1] gamma[k,m]) +
                            G342 gamma[i,k] gamma[m,m1]];
IndexSet[G44[k_,l_,m_,m1_], (G441 (gamma[k,m] gamma[l,m1] + gamma[k,m1] gamma[l,m]) +
                             G442 gamma[k,l] gamma[m,m1])];

IndexSet[G11[i_,j_,k_,l_,m_,m1_],(G111 gamma[i,j] gamma[k,l] gamma[m,m1] +
                                  G112 gamma[i,j] (gamma[k,m] gamma[l,m1] + gamma[k,m1] gamma[m,l]) +
                                  G113 (gamma[i,k] gamma[j,l] + gamma[i,l] gamma[j,k]) gamma[m,m1] +
                                  G113 (gamma[i,m] gamma[j,m1] + gamma[i,m1] gamma[j,m]) gamma[k,l] +
                                  G114 (gamma[i,k] gamma[j,m] gamma[l,m1] +
                                        gamma[i,k] gamma[j,m1] gamma[l,m] +
                                        gamma[i,l] gamma[j,m1] gamma[k,m] +
                                        gamma[i,l] gamma[j,m] gamma[k,m1] +
                                        gamma[i,m] gamma[j,k] gamma[m1,l] +
                                        gamma[i,m] gamma[j,l] gamma[m1,k] +
                                        gamma[i,m1] gamma[j,l] gamma[m,k] +
                                        gamma[i,m1] gamma[j,k] gamma[m,l]))+
                                 (A111 (gamma[i,k] gamma[j,m] gamma[l,m1] -
                                        gamma[j,k] gamma[i,m] gamma[l,m1] +
                                        gamma[i,k] gamma[j,m1] gamma[l,m] -
                                        gamma[j,k] gamma[i,m1] gamma[l,m] +
                                        gamma[i,l] gamma[j,m] gamma[k,m1] -
                                        gamma[j,l] gamma[i,m] gamma[k,m1] +
                                        gamma[i,l] gamma[j,m1] gamma[k,m] -
                                        gamma[j,l] gamma[i,m1] gamma[k,m]))];
];

undoAnsatz2nd :=
Module[{},
ansatz2ndMade = False;
G55[] =.;
G22[i_,j_] =.;
G26[i_,m_] =.;
G35[i_,k_] =.;
G45[k_,l_] =.;
G66[k_,m_] =.;
G12[i_,j_,k_,l_] =.;
G33[i_,j_,k_,m_] =.;
G16[i_,k_,l_,m_] =.;
G34[i_,k_,m_,m1_] =.;
G44[k_,l_,m_,m1_] =.;
G11[i_,j_,k_,l_,m_,m1_] =.;
];






makeMetricAnsatzZ4 :=
Module[{},
ansatzZ4Made = True;

IndexSet[G55Z4[], G551];

IndexSet[G22Z4[i_,j_],G221 gamma[i,j]];
IndexSet[G26Z4[i_,m_],G261 gamma[i,m]];
IndexSet[G35Z4[i_,-k_],G351 delta[i,-k]];
IndexSet[G45Z4[k_,l_],G451 gamma[k,l]];
IndexSet[G66Z4[k_,m_],G661 gamma[k,m]];

IndexSet[G12Z4[i_,j_,k_,l_],(G121 (gamma[i,k] gamma[j,l] + gamma[i,l] gamma[j,k]) +
                             G122 gamma[i,j] gamma[k,l])];
IndexSet[G33Z4[i_,j_,-k_,-m_],(G331 (delta[i,-k] delta[j,-m] + delta[i,-m] delta[j,-k]) +
                               G332 gamma[i,j] gamma[-k,-m]) +
                              (A331 (delta[i,-k] delta[j,-m] - delta[i,-m] delta[j,-k]))];
IndexSet[G16Z4[i_,k_,l_,m_],(G161 (gamma[i,k] gamma[m,l] + gamma[m,k] gamma[i,l]) +
                             G162 gamma[i,m] gamma[k,l])];
IndexSet[G34Z4[i_,-k_,m_,m1_], G341 (gamma[i,m] delta[-k,m1] + gamma[i,m1] delta[-k,m]) +
                                 G342 delta[i,-k] gamma[m,m1]];
IndexSet[G44Z4[k_,l_,m_,m1_], (G441 (gamma[k,m] gamma[l,m1] + gamma[k,m1] gamma[l,m]) +
                               G442 gamma[k,l] gamma[m,m1])];

IndexSet[G11Z4[i_,j_,k_,l_,m_,m1_],(G111 gamma[i,j] gamma[k,l] gamma[m,m1] +
                                    G112 gamma[i,j] (gamma[k,m] gamma[l,m1] + gamma[k,m1] gamma[m,l]) +
                                    G113 (gamma[i,k] gamma[j,l] + gamma[i,l] gamma[j,k]) gamma[m,m1] +
                                    G113 (gamma[i,m] gamma[j,m1] + gamma[i,m1] gamma[j,m]) gamma[k,l] +
                                    G114 (gamma[i,k] gamma[j,m] gamma[l,m1] +
                                          gamma[i,k] gamma[j,m1] gamma[l,m] +
                                          gamma[i,l] gamma[j,m1] gamma[k,m] +
                                          gamma[i,l] gamma[j,m] gamma[k,m1] +
                                          gamma[i,m] gamma[j,k] gamma[m1,l] +
                                          gamma[i,m] gamma[j,l] gamma[m1,k] +
                                          gamma[i,m1] gamma[j,l] gamma[m,k] +
                                          gamma[i,m1] gamma[j,k] gamma[m,l]))+
                                   (A111 (gamma[i,k] gamma[j,m] gamma[l,m1] -
                                          gamma[j,k] gamma[i,m] gamma[l,m1] +
                                          gamma[i,k] gamma[j,m1] gamma[l,m] -
                                          gamma[j,k] gamma[i,m1] gamma[l,m] +
                                          gamma[i,l] gamma[j,m] gamma[k,m1] -
                                          gamma[j,l] gamma[i,m] gamma[k,m1] +
                                          gamma[i,l] gamma[j,m1] gamma[k,m] -
                                          gamma[j,l] gamma[i,m1] gamma[k,m]))];
];

undoAnsatzZ4 :=
Module[{},
ansatzZ4Made = False;
G55Z4[] =.;
G22Z4[i_,j_] =.;
G26Z4[i_,m_] =.;
G35Z4[i_,-k_] =.;
G45Z4[k_,l_] =.;
G66Z4[k_,m_] =.;
G12Z4[i_,j_,k_,l_] =.;
G33Z4[i_,j_,-k_,-m_] =.;
G16Z4[i_,k_,l_,m_] =.;
G34Z4[i_,-k_,m_,m1_] =.;
G44Z4[k_,l_,m_,m1_] =.;
G11Z4[i_,j_,k_,l_,m_,m1_] =.;
];







TensorsDefined = True;
