(*********** auxiliary functions ***********)


(*** basis expansion of a matrix ***)
TensorDot[v_List, M_?MatrixQ, w_List] :=
Module[{i, j},
       ArrayFlatten[Table[M[[i, j]] Transpose[{v[[i]]}].{w[[j]]},
                          {i, Length[v]}, {j, Length[w]}]]];




(***** function to transform a matrix to block diagonal form by exchanging rows and columns if possible *****)
BlockForm[M_?MatrixQ] :=
Module[{elementsNeeded = {1},
        rowCounter, colCounter,
        size = 1,
        sizeOld = 0,
        subBlockForm},
       While[sizeOld<size,
             sizeOld=size;
             For[rowCounter = 1, rowCounter<=size, rowCounter++,
                 For[colCounter = 1, colCounter<=Length[M], colCounter++,
                     If[Not[MemberQ[elementsNeeded,colCounter]] &&
                        M[[elementsNeeded[[rowCounter]], colCounter]] =!= 0,
                        elementsNeeded = Join[elementsNeeded, {colCounter}]]
                    ];
                ];
             size = Length[elementsNeeded];
             For[colCounter = 1, colCounter<=size, colCounter++,
                 For[rowCounter = 1, rowCounter<=Length[M], rowCounter++,
                     If[Not[MemberQ[elementsNeeded,rowCounter]] &&
                        M[[elementsNeeded[[colCounter]], rowCounter]] =!= 0,
                        elementsNeeded = Join[elementsNeeded, {rowCounter}]]
                    ];
                ];
             size = Length[elementsNeeded]
            ];
       If[size<Length[M],
          subBlockForm = BlockForm[Part[M, Complement[Range[Length[M]],elementsNeeded],
                                           Complement[Range[Length[M]],elementsNeeded]]];
          {Join[elementsNeeded,
                Part[Complement[Range[Length[M]],elementsNeeded],subBlockForm[[1]]]],
           ArrayFlatten[{{Part[M, elementsNeeded,
                                  elementsNeeded],
                          Part[M, elementsNeeded,
                                  Complement[Range[Length[M]],elementsNeeded]]},
                         {Part[M, Complement[Range[Length[M]],elementsNeeded],
                                  elementsNeeded],
                          subBlockForm[[2]]}}]},
           {Range[size],M}]
      ];

BlockForm::"size" := "Error: The size `1` of the matrix does not match the size `2` of the premutation."
BlockForm[M_?MatrixQ, perm_List] :=
Module[{},
       If[Length[perm]=!=Length[M],
          Message[BlockForm::"size", Length[M], Length[perm]];
          Throw[Null]];
       Part[M, perm, perm]
      ];






(*** perform one step of the Gauss elimination algorithm ***)
GaussStep[M_?MatrixQ, n_, piv_] :=
Module[{mat = M,
        permL = IdentityMatrix[Length[M]],
        permR = IdentityMatrix[Length[M[[1]]]],
        i},

       permL[[n]] = UnitVector[Length[M], piv[[1]]];
       permL[[piv[[1]]]] = UnitVector[Length[M], n];

       permR[[n]] = UnitVector[Length[M[[1]]], piv[[2]]];
       permR[[piv[[2]]]] = UnitVector[Length[M[[1]]], n];

       mat = permL.M.permR;

       For[i = n + 1,
           i <= Length[M],
           i++,
           mat[[i]] = mat[[i]] - mat[[n]]/mat[[n, n]] mat[[i, n]]
          ];
       mat
      ];

(*** is an expression a nonzero number? ***)
NonzeroNumericQ[x_] := ((NumericQ[x]) && (x != 0));

(*** search in M for entries that satisfy nonZeroQ and perform Gauss elimination
     with the found pivot until no nonZero entry can be found ***)
PerformGSteps[M_?MatrixQ, nonZeroQ_] :=
Module[{modMatrix=M,
        foundNumber = True,
        stepCounter = 1,
        rowCounter, colCounter,
        pivot},
       While[foundNumber,
             foundNumber = False;
             rowCounter = stepCounter;
             While[(!foundNumber) && (rowCounter<=Length[modMatrix]),
                   colCounter = stepCounter;
                   While[(!foundNumber) && (colCounter<=Length[modMatrix[[1]]]),
                         If[nonZeroQ[modMatrix[[rowCounter,colCounter]]],
                            pivot={rowCounter,colCounter};
                            foundNumber = True];
                         colCounter++
                        ];
                   rowCounter++
                  ];
             If[foundNumber,
                modMatrix = GaussStep[modMatrix, stepCounter, pivot];
                stepCounter++];
            ];
       {stepCounter, modMatrix}
      ];


(*** to have less work with typing ***)
ToMakeRule[{Rule[lhs_, rhs_]}] := MakeRule[{lhs, rhs}, MetricOn->All];







