Needs["xAct`xTensor`"];

<<scripts/modified_Hamiltonian.m
<<scripts/PP_functions.m


(*** transformation canonical->Z4 ***)
SetOptions[extractPP,
           {ReshapeFunction ->
            Hold[ChristoffelToGradMetric[ChangeCovD[ChangeCurvature[#1]]]&]}];

(* dp = Up dK *)
pPP =
extractPP[{0,0,0}][{K,Theta,Z}][Simplification[NoScalar /@ ({pi[i, j], sigma[], rho[-i]} /. RuleCanontoZ4)]];
pPPMatrix =
Simplification[PowerExpand[PPMatrix[{0, 0, 0}, {K, Theta, Z}, {}, {i2, j2}][pPP]]];
pMatrix[i_, j_, i2_, j2_] := Evaluate@pPPMatrix;

(*** transformation Z4->canonical ***)
SetOptions[extractPP,
           {ReshapeFunction ->
            Hold[NoScalar[PowerExpand[ChristoffelToGradMetric[ChangeCovD[ChangeCurvature[#1]]]]] &]}];

KPP =
extractPP[{0,0,0}][{pi,sigma,rho}][Simplification[NoScalar /@ ({K[-i, -j], Theta[], Z[-i]} /. RuleZ4toCanon)]];
KPPMatrix =
Simplification[PowerExpand[PPMatrix[{0, 0, 0}, {pi, sigma, rho}, {}, {i2, j2}][KPP]]];
KMatrix[-i_, -j_, -i2_, -j2_] := Evaluate@KPPMatrix;





(*** inverse transformations ***)
(* UP^(-1) = DP *)
pPPMatrixInverse = PowerExpand[KPPMatrix /. RuleCanontoZ4];
(* DP^(-1) = UP *)
KPPMatrixInverse = PowerExpand[pPPMatrix /. RuleZ4toCanon];


KMatrixInverse[i_, j_, i2_, j2_] := Evaluate@KPPMatrixInverse;
pMatrixInverse[-i_, -j_, -i2_, -j2_] := Evaluate@pPPMatrixInverse;








(****************** GHG section ***********************)


(*** Minv, V, F, FT, M of the FOITSOIS system in canonical vars ***)
Options[blocksGHG] := {PostProcFunction -> Simplification};
blocksGHG[i_,j_,k_,l_,m_,m1_] :=
Module[{postProcFunction = ((PostProcFunction /. Options[blocksGHG]) /. {PostProcFunction -> ID})},
       If[recalcBlocksGHG =!= False,
          calculatedblocksGHG =
          postProcFunction[calcHamRhsPPMatrixBlocks[HS[] - Hadvection[] + HgaugeGHG[],
                                                    {2, 2, 2}, {gamma, alpha, beta},
                                                    {1, 1, 1}, {pi, sigma, rho},
                                                    {}, {},
                                                    {i1, i2}, {j1, j2}, {k1, k2}]];
          recalcBlocksGHG = False
         ];
       Return[Evaluate@(calculatedblocksGHG /. {i1->i,i2->j,j1->k,j2->l,k1->m,k2->m1})]
      ];

MinvGHG[-k_,-l_,-m_,-m1_] := blocksGHG[i,j,k,l,m,m1][[1]];
VGHG[i_,j_,k_,l_,m_,m1_] := blocksGHG[i,j,k,l,m,m1][[2]];
FGHG[i_,-k_,-l_,m_,m1_] := blocksGHG[i,j,k,l,m,m1][[3]];
FTGHG[i_,k_,l_,-m_,-m1_] := blocksGHG[i,j,k,l,m,m1][[4]];

MGHG[l1_, l2_, k1_, k2_] :=
{{(Sqrt[Detgamma[]] (gamma[k1, l2]*gamma[k2, l1] +
                     gamma[k1, l1]*gamma[k2, l2] -
                     gamma[k1, k2]*gamma[l1, l2]))/(4 alpha[]),
  -(Sqrt[Detgamma[]]*gamma[l1, l2])/(2*alpha[]^2),
  0},
 {-(Sqrt[Detgamma[]]*gamma[k1, k2])/(2*alpha[]^2),
  Sqrt[Detgamma[]]/alpha[]^3,
  0},
 {0, 0, -((Sqrt[Detgamma[]]*gamma[-k1, -l1])/alpha[]^3)}};



(*** Aij, Bi of the fully second order system ***)
Options[AijGHG] := {PostProcFunction -> Simplification};
AijGHG[i_, j_, -k_, -l_, m_, m1_] :=
Module[{l1,l2,i3,j3,k3,l3,
        postProcFunction = ((PostProcFunction /. Options[AijGHG]) /. {PostProcFunction -> ID})},
       If[recalcAijGHG =!= False,
          calculatedAijGHG =
          postProcFunction[MinvGHG[-j1,-j2,-l1,-l2].(
                            VGHG[i1,i2,l1,l2,k1,k2] -
                            FTGHG[i1,l1,l2,-i3,-j3].MGHG[i3,j3,k3,l3].FGHG[i2,-k3,-l3,k1,k2]/2 -
                            FTGHG[i2,l1,l2,-i3,-j3].MGHG[i3,j3,k3,l3].FGHG[i1,-k3,-l3,k1,k2]/2)] // Evaluate;
          recalcAijGHG = False
         ];
       Return[Evaluate@(calculatedAijGHG /. {i1->i, i2->j, j1->k, j2->l, k1->m, k2->m1})]
      ];

Options[BiGHG] := {PostProcFunction -> Simplification};
BiGHG[i_, -k_, -l_, m_, m1_] :=
Module[{l1,l2,i3,j3,
        postProcFunction = ((PostProcFunction /. Options[BiGHG]) /. {PostProcFunction -> ID})},
       If[recalcBiGHG =!= False,
          calculatedBiGHG =
          postProcFunction[FGHG[i1, -j1, -j2, k1, k2] +
                           MinvGHG[-j1,-j2,-l1,-l2].FTGHG[i1,l1,l2,-i3,-j3].MGHG[i3,j3,k1,k2]];
          recalcBiGHG = False
         ];
       Return[Evaluate@(calculatedBiGHG /. {i1->i, j1->k, j2->l, k1->m, k2->m1})]
      ];



(*** auxiliary matrices ***)
Options[H3GHG] := {PostProcFunction -> Simplification};
H3GHG[i_, j_, k_, l_, m_, m1_] :=
Module[{l1,l2,i3,j3,
        postProcFunction = ((PostProcFunction /. Options[H3GHG]) /. {PostProcFunction -> ID})},
       If[recalcH3GHG =!= False,
          calculatedH3GHG =
          postProcFunction[VGHG[i1,i2,j1,j2,k1,k2] -
                           FTGHG[i1,j1,j2,-l1,-l2].MGHG[l1,l2,i3,j3].FGHG[i2,-i3,-j3,k1,k2]/2 -
                           FTGHG[i2,j1,j2,-l1,-l2].MGHG[l1,l2,i3,j3].FGHG[i1,-i3,-j3,k1,k2]/2];
          recalcH3GHG = False
         ];
       Return[Evaluate@(calculatedH3GHG /. {i1->i, i2->j, j1->k, j2->l, k1->m, k2->m1})]
      ];

Options[H2GHG] := {PostProcFunction -> Simplification};
H2GHG[i_, -k_, -l_, m_, m1_] :=
Module[{l1,l2,
        postProcFunction = ((PostProcFunction /. Options[H2GHG]) /. {PostProcFunction -> ID})},
       If[recalcH2GHG =!= False,
          calculatedH2GHG =
          postProcFunction[MGHG[j1,j2,l1,l2].FGHG[i1, -l1, -l2, k1, k2] +
                           FTGHG[i1,j1,j2,-l1,-l2].MGHG[l1,l2,k1,k2]];
          recalcH2GHG = False
         ];
       Return[Evaluate@(calculatedH2GHG /. {i1->i, j1->k, j2->l, k1->m, k2->m1})]
      ];



(*** canonical candidates ***)
canonCandFOITSOISGHG[i_, j_, k_, l_, m_, m1_] :=
Module[{},
       Evaluate@ArrayFlatten[{{VGHG[i, j, k, l, m, m1], FTGHG[i, k, l, -m, -m1]},
                              {FGHG[j, -k, -l, m, m1], MinvGHG[-k, -l, -m, -m1]}}]
      ];

canonCand2ndGHG[i_, j_, k_, l_, m_, m1_] :=
Module[{},
       Evaluate@ArrayFlatten[{{H3GHG[i, j, k, l, m, m1], Table[0, {3}, {3}]},
                              {Table[0, {3}, {3}], MGHG[k, l, m, m1]}}]
      ];











(****************** general gauge section ***********************)



(*** Minv, V, F, FT, M of the FOITSOIS system in canonical vars ***)
Options[blocks] := {PostProcFunction -> Simplification};
blocks[i_,j_,k_,l_,m_,m1_] :=
Module[{postProcFunction = ((PostProcFunction /. Options[blocks]) /. {PostProcFunction -> ID})},
       If[recalcBlocks =!= False,
          calculatedblocks =
          postProcFunction[calcHamRhsPPMatrixBlocks[HS[] - Hadvection[] + HgaugeGHG[] + Hgauge[],
                                                    {2, 2, 2}, {gamma, alpha, beta},
                                                    {1, 1, 1}, {pi, sigma, rho},
                                                    {}, {},
                                                    {i1, i2}, {j1, j2}, {k1, k2}]];
          recalcBlocks = False
         ];
       Return[Evaluate@(calculatedblocks /. {i1->i,i2->j,j1->k,j2->l,k1->m,k2->m1})]
      ];

Minv[-k_,-l_,-m_,-m1_] := blocks[i,j,k,l,m,m1][[1]];
V[i_,j_,k_,l_,m_,m1_] := blocks[i,j,k,l,m,m1][[2]];
F[i_,-k_,-l_,m_,m1_] := blocks[i,j,k,l,m,m1][[3]];
FT[i_,k_,l_,-m_,-m1_] := blocks[i,j,k,l,m,m1][[4]];

(**** M in canonical variables ****)
RuleM =
{B1 -> Sqrt[Detgamma[]]/(4*alpha[]),
 B2 -> -((1 + 2*C1 + 2*C1^2 + 4*C4)*Sqrt[Detgamma[]])/(4*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]),
 B3 -> -((1 + 2*C1)*Sqrt[Detgamma[]])/(2*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]^2),
 B4 -> -((1 + 2*C1)*Sqrt[Detgamma[]])/(2*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]^2),
 B5 -> Sqrt[Detgamma[]]/((1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]^3),
 B6 -> -(Sqrt[Detgamma[]]/((1 + 2*C6)*alpha[]^3))};

M[i_, j_, k_, l_] :=
{{B1 (gamma[i, k] gamma[j, l] + gamma[j, k] gamma[i, l]) + B2 gamma[i, j] gamma[k, l],
  B3 gamma[i, j],
  0},
 {B4 gamma[k, l], B5, 0},
 {0, 0, B6 gamma[-i, -k]}} /. RuleM;


(* matrices in undensitized Z4 variables *)
MinvZ4[-i_, -j_, k_, l_] :=
Module[{k1, l1},
       Simplification[PowerExpand[Minv[-i, -j, -k1, -l1].pMatrix[k1, l1, k, l]]]];

VZ4[m_, m1_, -i_, -j_, k_, l_] :=
Module[{i2, j2},
       Simplification[PowerExpand[pMatrixInverse[-i, -j, -i2, -j2].V[m, m1, i2, j2, k, l]]]];

FZ4[m_, -i_, -j_, k_, l_] :=
Module[{},
       Simplification[PowerExpand[F[m, -i, -j, k, l]]]];

FTZ4[m_, -i_, -j_, k_, l_] :=
Module[{i2, j2, k1, l1},
       Simplification[PowerExpand[pMatrixInverse[-i, -j, -i2, -j2].
                                  FT[m, i2, j2, -k1, -l1].
                                  pMatrix[k1, l1, k, l]]]];

RuleMZ4 =
{B1 -> -1/(4*alpha[]),
 B2 -> (1 + 2*C1)^2/(8*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]),
 B3 -> -(1 + 2*C1)/(4*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]^2),
 B4 -> -(1 + 2*C1)/(4*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]),
 B5 -> 1/(2*(1 + 3*C1 + 3*C1^2 + 2*C4)*alpha[]^2),
 B6 -> (2*alpha[]^2 + 4*C6*alpha[]^2)^(-1)};

MZ4[-i_,-j_,k_,l_] :=
{{B1 (gamma[-i, k] gamma[-j, l] + gamma[-j, k] gamma[-i, l]) +
  B2 gamma[-i, -j] gamma[k, l],
  B3 gamma[-i, -j], 0},
 {B4 gamma[k, l], B5, 0},
 {0, 0, B6 gamma[-i, -k]}} /. RuleMZ4;


FOITSOISPSymbolZ4[s_?xTensorQ, -k_, -l_, m_, m1_] :=
ArrayFlatten[{{FZ4[j, -k, -l, m, m1] s[-j], MinvZ4[-k, -l, m, m1]},
              {VZ4[i, j, -k, -l, m, m1] s[-i] s[-j], FTZ4[i, -k, -l, m, m1] s[-i]}}];

(** fully 2nd order matrices in Z4 variables (must be the same in canon vars) **)
BiZ4[i_, -k_, -l_, m_, m1_] :=
Module[{k1,k2,l1,l2},
       FZ4[i, -k, -l, m, m1] +
       MinvZ4[-k, -l, k2, l2].
       FTZ4[i, -k2, -l2, k1, l1].
       MZ4[-k1, -l1, m, m1]
      ];

AijZ4[i_, j_, -k_, -l_, m_, m1_] :=
Module[{k1,k2,k3,l1,l2,l3},
       MinvZ4[-k,-l,k3,l3].(VZ4[i, j, -k3, -l3, m, m1] -
                            (FTZ4[i, -k3, -l3, k2, l2].
                             MZ4[-k2, -l2, k1, l1].
                             FZ4[j, -k1, -l1, m, m1])/2 -
                            (FTZ4[j, -k3, -l3, k2, l2].
                             MZ4[-k2, -l2, k1, l1].
                             FZ4[i, -k1, -l1, m, m1]/2))
      ];

PPMatrixZ42nd[i2_,-i_, j_, -k_, -l_, m_, m1_] :=
ArrayFlatten[{{Table[0, {3}, {3}],
               DiagonalMatrix[{delta[i2, -i] (delta[m, -k] delta[m1, -l] + delta[m1, -k] delta[m, -l])/2,
                               delta[i2, -i],
                               delta[i2, -i] delta[k, -m]}]},
              {AijZ4[i2, j, -k, -l, m, m1],
               BiZ4[i2, -k, -l, m, m1]}}];

PSymbolZ42nd[s_?xTensorQ, -k_, -l_, m_, m1_] :=
ArrayFlatten[{{Table[0, {3}, {3}],
               DiagonalMatrix[{(delta[m, -k] delta[m1, -l] + delta[m1, -k] delta[m, -l])/2,
                               1,
                               delta[k, -m]}]},
              {AijZ4[i2, j, -k, -l, m, m1]s[-i2]s[-j],
               BiZ4[i2, -k, -l, m, m1]s[-i2]}}];







(*** Aij, Bi of the fully second order system ***)
Options[Aij] := {PostProcFunction -> Hold[Simplification[#]&]};
Aij[i_, j_, -k_, -l_, m_, m1_] :=
Module[{l1,l2,i3,j3,k3,l3,
        postProcFunction = ((PostProcFunction /. Options[Aij]) /. {PostProcFunction -> ID})},
       If[recalcAij =!= False,
          calculatedAij =
          Evaluate[ReleaseHold[postProcFunction[
            Minv[-j1,-j2,-l1,-l2].(V[i1,i2,l1,l2,k1,k2] -
                                   FT[i1,l1,l2,-i3,-j3].M[i3,j3,k3,l3].F[i2,-k3,-l3,k1,k2]/2 -
                                   FT[i2,l1,l2,-i3,-j3].M[i3,j3,k3,l3].F[i1,-k3,-l3,k1,k2]/2)]]];
          recalcAij = False
         ];
       Return[Evaluate@(calculatedAij /. {i1->i, i2->j, j1->k, j2->l, k1->m, k2->m1})]
      ];

Options[Bi] := {PostProcFunction -> Simplification};
Bi[i_, -k_, -l_, m_, m1_] :=
Module[{l1,l2,i3,j3,
        postProcFunction = ((PostProcFunction /. Options[Bi]) /. {PostProcFunction -> ID})},
       If[recalcBi =!= False,
          calculatedBi =
          postProcFunction[F[i1, -j1, -j2, k1, k2] +
                           Minv[-j1,-j2,-l1,-l2].FT[i1,l1,l2,-i3,-j3].M[i3,j3,k1,k2]];
          recalcBi = False
         ];
       Return[Evaluate@(calculatedBi /. {i1->i, j1->k, j2->l, k1->m, k2->m1})]
      ];



(*** auxiliary matrices ***)
Options[H3] := {PostProcFunction -> Simplification};
H3[i_, j_, k_, l_, m_, m1_] :=
Module[{l1,l2,i3,j3,
        postProcFunction = ((PostProcFunction /. Options[H3]) /. {PostProcFunction -> ID})},
       If[recalcH3 =!= False,
          calculatedH3 =
          postProcFunction[V[i1,i2,j1,j2,k1,k2] -
                           FT[i1,j1,j2,-l1,-l2].M[l1,l2,i3,j3].F[i2,-i3,-j3,k1,k2]/2 -
                           FT[i2,j1,j2,-l1,-l2].M[l1,l2,i3,j3].F[i1,-i3,-j3,k1,k2]/2];
          recalcH3 = False
         ];
       Return[Evaluate@(calculatedH3 /. {i1->i, i2->j, j1->k, j2->l, k1->m, k2->m1})]
      ];

Options[H2] := {PostProcFunction -> Simplification};
H2[i_, -k_, -l_, m_, m1_] :=
Module[{l1,l2,
        postProcFunction = ((PostProcFunction /. Options[H2]) /. {PostProcFunction -> ID})},
       If[recalcH2 =!= False,
          calculatedH2 =
          postProcFunction[M[j1,j2,l1,l2].F[i1, -l1, -l2, k1, k2] +
                           FT[i1,j1,j2,-l1,-l2].M[l1,l2,k1,k2]];
          recalcH2 = False
         ];
       Return[Evaluate@(calculatedH2 /. {i1->i, j1->k, j2->l, k1->m, k2->m1})]
      ];



(*** canonical candidates ***)
canonCandFOITSOIS[i_, j_, k_, l_, m_, m1_] :=
Module[{},
       Evaluate@ArrayFlatten[{{V[i, j, k, l, m, m1], FT[i, k, l, -m, -m1]},
                              {F[j, -k, -l, m, m1], Minv[-k, -l, -m, -m1]}}]
      ];

canonCand2nd[i_, j_, k_, l_, m_, m1_] :=
Module[{},
       Evaluate@ArrayFlatten[{{H3[i, j, k, l, m, m1], Table[0, {3}, {3}]},
                              {Table[0, {3}, {3}], M[k, l, m, m1]}}]
      ];

loadPrevResults :=
Module[{},
       {recalcBlocksGHG, calculatedblocksGHG} = Evaluate@<<intermed_results/Ham_GHG_FOITSOIS_PP_Blocks.m;
       {recalcAijGHG, calculatedAijGHG, recalcBiGHG, calculatedBiGHG} =
            Evaluate@<<intermed_results/GHG_2nd_PP_Blocks.m;
       {recalcH3GHG, calculatedH3GHG, recalcH2GHG, calculatedH2GHG} =
            Evaluate@<<intermed_results/GHG_aux_matrices.m;
       {recalcBlocks, calculatedblocks} = Evaluate@<<intermed_results/Ham_FOITSOIS_PP_Blocks.m;
       {recalcAij, calculatedAij, recalcBi, calculatedBi} Evaluate@<<intermed_results/2nd_PP_Blocks.m;
       {recalcH3, calculatedH3, recalcH2, calculatedH2} Evaluate@<<intermed_results/aux_matrices.m;
       ProjPsZ4[-i_, -j_, k_, l_] := Evaluate@<<intermed_results/ProjPsZ4ijkl;
       StandardDecompBlockAnsatzCandZ4[i_, j_, k_, l_, m_, m1_] :=
           Evaluate@<<intermed_results/StandardDecompBlockAnsatzCandZ4OfMetricijklmm1;
       StandardGssZ4[k_, l_, m_, m1_] :=
           Evaluate@Take[StandardDecompBlockAnsatzCandZ4[i, j2, k, l, m, m1], {1, 6}, {1, 6}];
       DecomposedStandardDecompBlockAnsatzCandZ4[i_, j_, k_, l_, m_, m1_] :=
           Evaluate@<<intermed_results/DecomposedStandardDecompBlockAnsatzCandZ4OfMetricijklmm1;
       DecomposedStandardGssZ4[k_, l_, m_, m1_] :=
           Evaluate@Take[DecomposedStandardDecompBlockAnsatzCandZ4[i, j, k, l, m, m1], {1, 14}, {1, 14}];
      ];




