Needs["xAct`xTensor`"];

(******** definitions of tensors **********)
If[TensorsDefined =!= True,
   <<scripts/Tensor_Definitions.m];

(******* constraints *********)
RuleHt := RuleHt =
MakeRule[{HamC[],
          Power[Detgamma[],-1/2] *
          ((gamma[-l,-i]gamma[-m,-j]pi[l, m]pi[i,j] -
            gamma[-i,-j]gamma[-l,-m]pi[i,j]pi[l, m]/2 -
            Detgamma[] RicciScalar[D3][]))
         }
        ];
RuleHi := RuleHi =
Evaluate@MakeRule[{MomC[-i], -2 gamma[-i,-j] D3[-k][pi[j,k]]},
                  MetricOn->All];

(***** physical Hamiltonian *****)
IndexSet[HS[],
         (alpha[] HamC[] + beta[i] MomC[-i]) /. Flatten[{RuleHi, RuleHt}]
        ];
(***** these advection terms modify the equations of motion,
       but in the principal part they just subtract beta*identity *****)
IndexSet[Hadvection[],
         pi[i,j] beta[k] PD[-k][gamma[-i, -j]] +
         sigma[] beta[k] PD[-k][alpha[]] +
         rho[-i] beta[k] PD[-k][beta[i]]
        ];

(***** Gauge hamiltonian *****)
IndexSet[HgaugeGHG[],
         - alpha[]^2 (gamma[-i, -i1] pi[i, i1] sigma[])/(2 Sqrt[Detgamma[]]) +
         alpha[]^3 sigma[]^2/(8 Sqrt[Detgamma[]]) +
         beta[i] sigma[] D3[-i][alpha[]] +
         rho[-i] (-alpha[]^3 rho[-i1] gamma[i, i1]/(2 Sqrt[Detgamma[]]) +
                   alpha[] (alpha[] ChristoffelD3[i, -i1, -i2] gamma[i1, i2] -
                            gamma[i, i1] D3[-i1][alpha[]])) +
         beta[i] rho[-i1] PD[-i][beta[i1]]
        ];

IndexSet[Hgauge[], 
         - C1 Power[alpha[], 2] gamma[-i, -j] pi[i, j] sigma[] Power[Detgamma[], -1/2] +
         C4 Power[alpha[], 3] Power[sigma[], 2] Power[Detgamma[], -1/2] -
         C7/2 alpha[] gamma[j, k] beta[i] PD[-i][gamma[-j, -k]] sigma[] +
         C7 alpha[] D3[-i][beta[i]] sigma[] +
         C2 alpha[]^2 Christoffel[D3][i, -j, -k] gamma[j, k] rho[-i] +
         C3 alpha[]^2 Christoffel[D3][i, -i, -j] gamma[j, k] rho[-k] -
         C5 alpha[] gamma[i, j] D3[-j][alpha[]] rho[-i] -
         C6 alpha[]^3 rho[-i] rho[-j] gamma[i, j] Power[Detgamma[], -1/2]
        ];

IndexSet[Hfull[],
         Simplification[(HS[]+ HgaugeGHG[] + Hgauge[])/. Flatten[{RuleHi, RuleHt}]]
        ];




(***** function to calculate the "time weight" *******)
TimeWeight[sum_Plus] :=
Module[{list = TimeWeight /@ (List@@sum)},
       If[Length[Union[list]]>1, Throw[Null]];
       list[[1]]
      ];
TimeWeight[prod_Times] := Plus @@ (TimeWeight/@ (List @@ prod));
TimeWeight[Power[expr_, x_]] := x TimeWeight[expr];
TimeWeight[d_?CovDQ[_][expr_]] := TimeWeight[expr];
TimeWeight[x_?ConstantQ] := 0;

TimeWeight[gamma[__]] := 0;
TimeWeight[Detgamma[]] := 0;
TimeWeight[pi[__]] := 0;
TimeWeight[alpha[]] := 1;
TimeWeight[sigma[]] := -1;
TimeWeight[beta[_]] := 1;
TimeWeight[rho[_]] := -1;
TimeWeight[ChristoffelD3[__]] := 0;
TimeWeight[RicciScalarD3[]] := 0;
TimeWeight[RicciD3[__]] := 0;
TimeWeight[RiemannD3[__]] := 0;

